/* functions for communicating with the pololu ssc over a serial port */

#include <stdio.h>   /* Standard input/output definitions */
#include <string.h>  /* String function definitions */
#include <unistd.h>  /* UNIX standard function definitions */
#include <fcntl.h>   /* File control definitions */
#include <errno.h>   /* Error number definitions */
#include <termios.h> /* POSIX terminal control definitions */
#include <sys/select.h>
#include "ssc.h"

/* write a message to the SSC */
void sendCommand(ssc *s, char id, char cmd, char servo, char data) {
  char buf[5];
  snprintf(buf,5,"%c%c%c%c%c",0x80,id,cmd,servo,data);
  write(s->fd,buf,5);
}

void sendCommand6(ssc *s, char controller, char cmd, char servo, char data1, char data2) {
  char buf[6];
  servo += controller*16;
  snprintf(buf,6,"%c%c%c%c%c%c",0x80,0x01,cmd,servo,data1,data2);
  write(s->fd,buf,6);
}

int enable(ssc *s, int controller, int servo, int enabled) {
  if(!s->connected) { return 0; }

  if(enabled) {
    sendCommand(s,(char)controller,0,(char)servo,(char)0x40);
    printf("enabled %i\n",servo); fflush(stdout);
  } else {
    sendCommand(s,(char)controller,0,(char)servo,0x00);
    printf("disabled %i\n",servo); fflush(stdout);
  }

}

int stepTo(ssc *s, int controller, int servo, int pos) {
  if(!s->connected) { return 0; }

  sendCommand6(s,(char)controller,4,(char)servo,(char)(pos/128),(char)(pos%128));

  return 1;
}

void endSession(ssc *s) {
  if(!s->connected) return;
  close(s->fd);
}

int connectSsc(ssc *s) {
  struct termios options;
  int count,ret;

  s->connected=0; /* reset in case we fail to connect */

  s->fd = open("/dev/ttyS1", O_RDWR | O_NOCTTY);
  if (s->fd == -1) return 0;
  else fcntl(s->fd, F_SETFL, 0);

  tcgetattr(s->fd, &options);

  /* go to 9600 baud */
  cfsetispeed(&options, B9600);
  cfsetospeed(&options, B9600);

  options.c_cflag |= (CLOCAL | CREAD); /* enable */

  options.c_cflag &= ~PARENB; /* 8N1 */
  options.c_cflag &= ~CSTOPB;
  options.c_cflag &= ~CSIZE;
  options.c_cflag |= CS8;

  /* set all of the options */
  tcsetattr(s->fd, TCSANOW, &options);

  s->connected=1;
  return 1;
}

const char *getId(ssc *s) {
  return s->idstring;
}
